from typing import Optional

from genx_ai_camera_panel.db import DB
from onvif.onvif_client import OnvifClient, WsDiscoveryClient


class OnvifWrapper:
    db: DB

    def __init__(self) -> None:
        self.db = DB()

    def scan(
        self,
        username: Optional[str],
        password: Optional[str],
    ):
        """
        Scan for available ONVIF devices.

        Returns:
            A list of dictionaries, each containing the device's IP address,
            port, and protocol (TCP or UDP).
        """

        wsd_client = WsDiscoveryClient()
        nvts = wsd_client.search()

        for nvt in nvts:
            self.scan_device(
                nvt.ip_address,
                nvt.port,
                username,
                password,
            )

        wsd_client.dispose()

    def scan_device(
        self,
        ip_address: str,
        port: int,
        username: Optional[str],
        password: Optional[str],
    ):
        """
        Scan a single ONVIF device.
        """
        try:
            print(f"[*] Scanning device: {ip_address}:{port}")
            onvif_client = OnvifClient(ip_address, port, username, password)

            hostname = onvif_client.get_hostname() or {}
            device_information = onvif_client.get_device_information()
            if device_information is None:
                return

            manufacturer = device_information["Manufacturer"] or ""
            model = device_information["Model"] or ""
            firmware_version = device_information["FirmwareVersion"] or ""
            serial_number = device_information["SerialNumber"] or ""
            hardware_id = device_information["HardwareId"] or ""

            profile_tokens = onvif_client.get_profile_tokens()
            video_configurations = onvif_client.get_video_encoder_configurations()

            for index, profile_token in enumerate(profile_tokens):
                stream_url = onvif_client.get_streaming_uri(profile_token)
                video_configuration = video_configurations[index]
                resolution = f"{video_configuration['Resolution']['Width']}x{video_configuration['Resolution']['Height']}"
                fps = video_configuration["RateControl"]["FrameRateLimit"]
                bitrate = video_configuration["RateControl"]["BitrateLimit"]
                encoding = video_configuration["Encoding"]

                self.db.add_camera(
                    {
                        "ip_address": ip_address,
                        "port_number": port,
                        "username": username,
                        "password": password,
                        "stream_url": stream_url,
                        "profile_token": profile_token,
                        "manufacturer": manufacturer,
                        "model": model,
                        "firmware_version": firmware_version,
                        "serial_number": serial_number,
                        "hardware_id": hardware_id,
                        "resolution": resolution,
                        "fps": fps,
                        "bitrate": bitrate,
                        "encoding": encoding,
                        "hostname": (hostname["Name"] or ""),
                    }
                )
        except Exception as exception:
            print(exception)
